####################################################################
### Example code for Estimating Tree, Forest, and Main Effects 
### Logistic Regression Models using the NHIS sample data referenced 
### in the Survey Practice article. 
###
### Outcome variable:  newrespond20
###
### Trent D. Buskirk, May 30, 2017.
####################################################################
rm(list=ls())

setwd('<< INSERT WORKING DIRECTORY PATH >>')
load('SPData2.Rda')

################################################################# 
###  Step 0:   Create the training and test sets to be used in  #              
###  the simple cross validation                                #
#################################################################

## rename the levels of the educ3 variable to facilitate printing 
## of the final tree models.
levels(SPData2$educ3)<-c("HSorLess", "Some Col", "BSorMore")
levels(SPData2$newrespond20)<-c("No", "Yes")

## create simple training and testing set based on pre-defined 
##folds of the dataset that were randomly assigned.
SPDtrain<-SPData2[SPData2$fold<=10,]

# fold 11 represents rougly 15% of the total data set and will 
# form our "test" set
SPDtest<-SPData2[SPData2$fold>10,]

# removing extra variables that won't be included in the models
SPDtrain<-SPDtrain[,c(1:5,7:9, 11:13,30)]
SPDtest<-SPDtest[,c(1:5,7:9, 11:13,30)]

SPDtrain<-data.frame(SPDtrain[2:12], SPDtrain$id)
names(SPDtrain)[12]<-c("id")
SPDtest<-data.frame(SPDtest[2:12], SPDtest$id)
names(SPDtest)[12]<-c("id")

############################################################## 
#######       STEP 1:  Estimate the tree model          ###### 
##############################################################
require(rpart)
require(randomForest)

set.seed(5312017)
nr2tree<-rpart(newrespond20~., data=SPDtrain[,-c(13)], control=rpart.control(cp=0.0005))
printcp(nr2tree)
plotcp(nr2tree)

# Based on the "One Standard Error" rule, the largest value of cp 
# that has an error that is within one standard error of the smallest 
# error is .0022.  We will use the final tree with this value of CP.
# we can either prune the tree, or grow it and specify the value of cp. 

nr2tree1<-prune.rpart(nr2tree, cp=0.0022)

#############################################################
########        STEP 2: PLot the TREE MODEL          ######## 
#############################################################
require(rpart.plot)
rpart.plot(nr2tree1, box.palette="BuRd",
branch.lty=1, branch.lwd=3, branch.col="darkgreen", type=1,shadow.col="white", nn=TRUE, cex=.95)

prop.table(table())

require("partykit")
tree.2<- as.party(nr2tree1)
plot(tree.2, gp = gpar(fontsize = 9.5))

names(tree.2)
tree.2$fitted[1:10,]

# enhancing the plot 
# see:  http://stackoverflow.com/questions/33314187/partykit-displaying-terminal-node-percentile-values-above-terminal-node-boxplot 

tab<-prop.table(table(tree.2$fitted[,1], tree.2$fitted[,2]), margin=1)[,2]
tab<-round(tab,2)
mlab <- function(id, nobs) paste("Rspd%=", tab[id])
plot(tree.2, tp_args = list(mainlab = mlab), gp = gpar(fontsize = 6.5, col="darkblue", lwd=2))

##############################################################
###### STEP 3: Assess Model Performance on the test set  #####  ##############################################################

#  Since the outcome variable is categorical we will compute a 
#  confusion matrix that can be used to compute relevant accuracy 
#  measures such as the Sensitivity,  Specificity, Percent 
#  Correctly Classified etc.

require(caret)
require(pROC)

# now check how well the tree model performed on the test set
nr2treepreds<-predict(nr2tree1, SPDtest[,-c(12,13)],type="class")

# get predicted probabilities for use in computing AUC 
nr2treepreds2<-predict(nr2tree1, SPDtest[,-c(12,13)])[,2]
# Compute Area Under the Receiver Operating Characteristic Curve
auc(SPDtest$newrespond20,nr2treepreds2)
# under the curve: 81.93
roc(SPDtest$newrespond20,nr2treepreds2, plot=TRUE, add=TRUE,col=1, lwd=2.5, lty=5)

nr2treeconfuse<-confusionMatrix(nr2treepreds, SPDtest$newrespond20, positive=c("Yes"))
nr2treeconfuse

# cross validated estimate of the Percentage Correctly Classified 
# for the tree model: 78.32%
# cross validated estimate of the Sensitivity for the tree model:  
# 57.49% (true positive rate)
# cross validated estimate of the Specificity for the tree model: 
# 92.12% (true negative rate) 
# overall balanced accuracy for the tree model (i.e. average of 
# true negative and true positive rate): 74.81%

#############################################################
########### Main Effects Logistic Regression Model for   #### 
#############################################################

nr2LR <- glm(newrespond20 ~ age + educ3 + hispanic2 + as.factor(incgrp4) + telstat+
  ratcat2 + region + sex + wborace + wrkcata, 
  family=binomial(logit), data=SPDtrain[,-c(12)])
summary(nr2LR)
nr2treepredsLR<-predict(nr2LR, SPDtest[,-c(11,12)], type="response")
nr2treepredsLRfinal<-as.numeric((nr2treepredsLR>=.5))
nr2treepredsLRfinal<-as.factor(nr2treepredsLRfinal)
levels(nr2treepredsLRfinal)<-c("No", "Yes")

# Area Under the Curve Calculations
auc(SPDtest$newrespond20,nr2treepredsLR)
# under the curve: 74.22

nr2treeconfuseLR<-confusionMatrix(nr2treepredsLRfinal, SPDtest$newrespond20, positive=c("Yes"))
nr2treeconfuseLR

# cross validated estimate of the Percentage Correctly Classified
# for the LR model: 69.82%
# cross validated estimate of the Sensitivity for the LR model:  
# 49.65% (true positive rate)
# cross validated estimate of the Specificity for the LR model: 
# 83.2% (true negative rate)
# overall balanced accuracy for the LR model (i.e. average of 
# true negative and true positive rate): 66.42%

############################################################### 
#########             Random Forest Models             ########
###############################################################

###############################################################
#######         STEP 1: Create the forest model         ####### 
###############################################################

require(randomForest)
set.seed(5312017)
nr2RF0<-randomForest(newrespond20~., data=SPDtrain[,-c(12)], importance=TRUE, na.action=na.roughfix, ntree=2000)
summary(nr2RF0)
# this will plot the estimated OOB error in the forest as a 
# function of the number of trees
plot(nr2RF0, lwd=2.5)
# we will take ntree=1000 as adding more trees beyond this 
# point does not add to accuracy of the model.

set.seed(622017)
nr2RF<-randomForest(newrespond20~., data=SPDtrain[,-c(12)], importance=TRUE, na.action=na.roughfix, ntree=1001)
summary(nr2RF)
varImpPlot(nr2RF, col=4, pch=16, cex=1.25)

################################################################# 
#######    Step 2:   Apply the forest model to the test data ####
#################################################################

# get predicted probabilities for use in computing the confusion matrix 
nr2predsRF1<-predict(nr2RF, newdata=SPDtest[,-c(12)], type="response")

# get predicted probabilities for use in computing AUC 
nr2predsRF2<-predict(nr2RF, newdata=SPDtest[,-c(11,12)], type="prob")[,2]

# Compute Area Under the Receiver Operating Characteristic Curve
auc(SPDtest$newrespond20,nr2predsRF2)
# Area under the curve: 83.80
nr2treeconfuseRF<-confusionMatrix(nr2predsRF1, SPDtest$newrespond20,  positive=c("Yes"))
nr2treeconfuseRF

# cross validated estimate of the Percentage Correctly Classified 
# for the Random Forest model: 78.53%
# cross validated estimate of the Sensitivity for the Random 
# Forest model:  60.30% (true positive rate)
# cross validated estimate of the Specificity for the Random 
# Forest model: 90.61% (true negative rate) 
# overall balanced accuracy for the tree model (i.e. average of 
# true negative and true positive rate): 75.46%

##plot ROC curves comparing the three models
roc(SPDtest$newrespond20,nr2treepredsLR, plot=TRUE, col="blue", cex.axis=1.25, cex.lab=1.15, lwd=3.5, lty=3)
roc(SPDtest$newrespond20,nr2predsRF2, plot=TRUE, add=TRUE, col=2, lwd=2.5)
roc(SPDtest$newrespond20,nr2treepreds2, plot=TRUE, add=TRUE,col=1, lwd=2.5, lty=5)
legend(.5, .2, legend=c("Logistic Regression", "Tree", "Random Forest"), col=c("blue", 1, 2), lty=c(3,5,1), lwd=2.5, cex=1.2)
